#include "string.h"
#include "util/atomic.h"

#include "cdelay.h"

#include "eep_tinymega.h"

uint16_t eepromaddress;

uint8_t EEP_ReadBegin(uint16_t AAddress)
{
  //set address
  eepromaddress=AAddress;

  //return ok
  return 1;
}

uint8_t EEP_ReadByte(uint8_t *AByte)
{
  uint8_t c1;

  //wait up to 10 milliseconds for EEPROM to not be busy 
  for (c1=0;EECR & _BV(EEPE);c1++)
    {
      if (c1==250)
        return 0;

      Delay_us(40);
    }

  //set and increment address
  EEAR=eepromaddress++;

  //request read
  EECR|=_BV(EERE);

  //read result
  *AByte=EEDR;

  //return ok
  return 1;
}

uint8_t EEP_ReadBlock(uint16_t ALength, uint8_t *ABlock)
{
  uint8_t c1;

  //iterate length  
  while (ALength--)
    {
      //read byte
      if (!EEP_ReadByte(&c1))
        return 0;

      //set byte in block
      *ABlock=c1;

      //increment block pointer
      ABlock++;
    }
    
  //return ok
  return 1;
}

uint8_t EEP_ReadEnd()
{
  //nothing to do for internal EEPROM

  //return ok
  return 1;
}

uint8_t EEP_WriteBegin(uint16_t AAddress)
{
  //set address
  eepromaddress=AAddress;

  //return ok
  return 1;
}

uint8_t EEP_WriteByte(uint8_t AByte)
{
  uint8_t existing;

  //wait for not busy, set and increment address, and get existing value
  if (!EEP_ReadByte(&existing))
    return 0;

  //if already set the same there is nothing to do
  if (existing==AByte)
    return 1;

  //can we write without erasing first
  existing=AByte & existing;

  if (existing==AByte)
    {
      //write only
      EECR|=_BV(EEPM1);
      EECR&=~_BV(EEPM0);
      goto perform;
    }

  //we must erase, can we erase only without writing
  if (AByte==0xFF)
    {
      //erase only
      EECR|=_BV(EEPM0);
      EECR&=~_BV(EEPM1);
      goto perform;
    }

  //we must erase and write
  EECR&=~(_BV(EEPM0)|_BV(EEPM1));

  //perform operation
  perform:

  EEDR=AByte;

  ATOMIC_BLOCK(ATOMIC_FORCEON)
    {
      EECR |= _BV(EEMPE);
      EECR |= _BV(EEPE);
    }

  //return ok
  return 1;
}

uint8_t EEP_WriteBlock(uint16_t ALength, uint8_t *ABlock)
{
  //iterate length
  while (ALength--)
    {
      //write byte
      if (!EEP_WriteByte(*ABlock))
        return 0;

      //increment block pointer
      ABlock++;
    }

  //return ok
  return 1;
}

uint8_t EEP_WriteEnd()
{
  //nothing to do for internal EEPROM

  //return ok
  return 1;
}

