#include "string.h"
#include "avr/pgmspace.h"
#include "util/crc16.h"
#include "util/atomic.h"
#include "util/twi.h"

#include "cdelay.h"
#include "ddrpin.h"

#include "twi_bitbang.h"

#include "twi_bitbang_settings.h"

void TWI_Init()
{
  //sda input no pullup (important because bit bang code switches pin direction to assert low)
  DDR(EEPROM_I2C_SDA_PORT)&=~_BV(EEPROM_I2C_SDA_PIN);
  EEPROM_I2C_SDA_PORT&=~_BV(EEPROM_I2C_SDA_PIN);

  //scl input no pullup (important because bit bang code switches pin direction to assert low)
  DDR(EEPROM_I2C_SCL_PORT)&=~_BV(EEPROM_I2C_SCL_PIN);
  EEPROM_I2C_SCL_PORT&=~_BV(EEPROM_I2C_SCL_PIN);
}

void TWI_Term()
{
}

uint8_t TWI_Start()
{
  //start
  if ((PIN(EEPROM_I2C_SCL_PORT) & _BV(EEPROM_I2C_SCL_PIN))==0)
    {
      DDR(EEPROM_I2C_SDA_PORT)&=~_BV(EEPROM_I2C_SDA_PIN);
      __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

      DDR(EEPROM_I2C_SCL_PORT)&=~_BV(EEPROM_I2C_SCL_PIN);
      __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);
    }

  DDR(EEPROM_I2C_SDA_PORT)|=_BV(EEPROM_I2C_SDA_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  DDR(EEPROM_I2C_SCL_PORT)|=_BV(EEPROM_I2C_SCL_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  return 1;
}

uint8_t TWI_Stop()
{
  DDR(EEPROM_I2C_SDA_PORT)|=_BV(EEPROM_I2C_SDA_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  DDR(EEPROM_I2C_SCL_PORT)&=~_BV(EEPROM_I2C_SCL_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  DDR(EEPROM_I2C_SDA_PORT)&=~_BV(EEPROM_I2C_SDA_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  return 1;
}

void TWI_BitWrite(uint8_t ABit)
{
  if (ABit)
    DDR(EEPROM_I2C_SDA_PORT)&=~_BV(EEPROM_I2C_SDA_PIN);
  else DDR(EEPROM_I2C_SDA_PORT)|=_BV(EEPROM_I2C_SDA_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  DDR(EEPROM_I2C_SCL_PORT)&=~_BV(EEPROM_I2C_SCL_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  DDR(EEPROM_I2C_SCL_PORT)|=_BV(EEPROM_I2C_SCL_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);
}

//returns the read bit, 1=high, 0=low
uint8_t TWI_BitRead()
{
  uint8_t c1;

  DDR(EEPROM_I2C_SDA_PORT)&=~_BV(EEPROM_I2C_SDA_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  DDR(EEPROM_I2C_SCL_PORT)&=~_BV(EEPROM_I2C_SCL_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  c1=(PIN(EEPROM_I2C_SDA_PORT) & _BV(EEPROM_I2C_SDA_PIN));
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  DDR(EEPROM_I2C_SCL_PORT)|=_BV(EEPROM_I2C_SCL_PIN);
  __builtin_avr_delay_cycles(EEPROM_BB_QCLOCK_DELAY_CYCLES);

  return c1;
}

//returns acknowledgement, 1=ack, 0=nack
uint8_t TWI_Write(uint8_t AData)
{
  uint8_t c1;

  for (c1=0;c1<8;c1++)
    {
      TWI_BitWrite(AData & _BV(7));
      AData<<=1;
    }

  return !TWI_BitRead();
}

uint8_t TWI_Read(uint8_t AAck, uint8_t *AData)
{
  uint8_t c1,c2;

  for (c1=0,c2=0;c1<8;c1++)
    {
      c2<<=1;
      if (TWI_BitRead())
        c2|=_BV(0);
    }

  TWI_BitWrite(AAck?0:1);

  *AData=c2;

  return 1;
}

