#include "string.h"
#include "avr/pgmspace.h"
#include "util/crc16.h"
#include "util/atomic.h"
#include "util/twi.h"

#include "cdelay.h"
#include "ddrpin.h"

#include "twi_tinymega.h"

#include "twi_tinymega_settings.h"

void TWI_Init()
{
  //sda input no pullup
  DDR(EEPROM_I2C_SDA_PORT)&=~_BV(EEPROM_I2C_SDA_PIN);
  EEPROM_I2C_SDA_PORT&=~_BV(EEPROM_I2C_SDA_PIN);

  //scl input no pullup
  DDR(EEPROM_I2C_SCL_PORT)&=~_BV(EEPROM_I2C_SCL_PIN);
  EEPROM_I2C_SCL_PORT&=~_BV(EEPROM_I2C_SCL_PIN);

  //enable twi
  TWBR=EEPROM_I2C_TWI_BIT_RATE;
  TWSR=0;
  TWCR=_BV(TWEN);
}

void TWI_Term()
{
  //disable twi
  TWCR=0;
  TWSR=0;
  TWBR=0;
}

uint8_t TWI_Start()
{
  uint8_t c1;

  //start
  TWCR=_BV(TWINT) | _BV(TWSTA) | _BV(TWEN);

  //wait
  while (!(TWCR & _BV(TWINT)))
    ;

  //return
  c1=TWSR & TW_STATUS_MASK;
  return (c1==TW_START || c1==TW_REP_START);
}

uint8_t TWI_Stop()
{
  //stop
  TWCR=_BV(TWINT) | _BV(TWSTO) | _BV(TWEN);

  //return
  return (TWSR & TW_STATUS_MASK)==TW_NO_INFO;
}

//returns acknowledgement, 1=ack, 0=nack
uint8_t TWI_Write(uint8_t AData)
{
  uint8_t c1;

  //set data
  TWDR=AData;

  //write
  TWCR=_BV(TWINT) | _BV(TWEN);

  //wait
  while (!(TWCR & _BV(TWINT)))
    ;

  //return
  c1=TWSR & TW_STATUS_MASK;
  return (c1==TW_MT_SLA_ACK || c1==TW_MT_DATA_ACK || c1==TW_MR_SLA_ACK || c1==TW_MR_DATA_ACK);
}

uint8_t TWI_Read(uint8_t AAck, uint8_t *AData)
{
  uint8_t c1;

  //read
  TWCR = (AAck?_BV(TWEA):0) | _BV(TWINT) | _BV(TWEN);

  //wait
  while (!(TWCR & _BV(TWINT)))
    ;

  *AData=TWDR;

  //return
  c1=TWSR & TW_STATUS_MASK;
  return (c1==TW_MT_DATA_ACK || c1==TW_MT_DATA_NACK || c1==TW_MR_DATA_ACK || c1==TW_MR_DATA_NACK);
}


